#!/usr/bin/python

from clicore.core import Cli, CliItem, CliSysPathItem
from clicore.clihelper import CliHelper


class MyCli(Cli):
    def __init__(self):
        Cli.__init__(self, ".my_cli.history", "Hello everybody")
        self.set_prompt("MyCli> ")

        self.register_item(CliItem("my_item"))
        self.register_item(CliItem("item", None, None, True, [CliSysPathItem()]))
        self.register_item(CliItem("bla", self.bla))
        self.register_item(CliItem("foo", self.foo))
        self.register_item(CliHelper.create_help_item(self))

    @Cli.obs.on("initialized")
    def initialized(self):
        print "on_initialized"

    @Cli.obs.on("before_start")
    def before_start(self):
        print "before_start"

    @Cli.obs.on("before_stop")
    def before_stop(self):
        print "before_stop"

    def bla(self, item, args, line_input):
        """
            arguments: <bla> [--foo]
            description: this command echoes just bla
        """
        print "bla"

    def foo(self, item, args, line_input):
        """some stupid command with wrong help"""
        print "foooo"


if __name__ == "__main__":
    my_cli = MyCli()
    my_cli.start()



# -*- coding: utf-8 -*-

from clicore.core import CliItem

import sys


class CliHelper(object):
    @classmethod
    def create_help_item(cls, cli):
        if not hasattr(cli, "get_items"):
            raise TypeError("first argument must have a 'get_items()' method")

        def help_item(item, args, line_input):
            items = cli.get_items()

            max_len_name = max([len(i.get_name()) for i in items])
            max_len_args = max([len(i.get_help()["arguments"]) for i in items])
            max_len_desc = max([len(i.get_help()["description"]) for i in items])

            indentation = "    "
            spaces = "      "
            for i in cli.get_items():
                name = i.get_name()
                help = i.get_help()
                sys.stdout.write(indentation + name + " " * (max_len_name - len(name)) + spaces)
                sys.stdout.write(help["arguments"] + " " * (max_len_args - len(help["arguments"])) + spaces)
                sys.stdout.write(help["description"] + " " * (max_len_desc - len(help["description"])) + "\n")

        return CliItem("help", help_item)



# -*- coding: utf-8 -*-

import os
import readline
from observable import Observable
from colorful import Colorful

from clicore.cliitem import CliItem, CliSysPathItem


class Cli(object):
    obs = Observable()

    def __init__(self, history_file=None, welcome_text=None, use_location=True):
        # configure readline
        readline.set_completer(self._complete)
        readline.parse_and_bind("tab: complete")

        self._history_file = history_file
        self._welcome_text = welcome_text

        self._started = False
        self._matches = []
        self._items = []

        self._prompt = "Cli> "

        # register observable events
        self.obs.register_event("initialized")  # triggered after Cli.__init__
        self.obs.register_event("before_start")  # triggered before cli loop is started
        self.obs.register_event("raw_input_entered")  # triggered after an input is entered
        self.obs.register_event("before_stop")  # triggered before cli loop is stopped
        self.obs.register_event("after_stop")  # triggered after cli loop is stopped

        # trigger 'on.initialized' event
        self.obs.trigger("initialized", self)

    def start(self):
        if self._history_file is not None:
            try:
                readline.read_history_file(os.path.expanduser(self._history_file))
            except IOError:  # occures if history file does not exist yet
                pass

        # trigger 'on.before_start' event
        self.obs.trigger("before_start", self)

        if self._welcome_text:
            print(self._welcome_text)

        self._started = True
        while self._started:
            try:
                line_input = raw_input(self._prompt)
                # trigger 'on.raw_input_entered' event
                self.obs.trigger("raw_input_entered", self)
                if not line_input:
                    continue
            except EOFError:  # occures when Ctrl+D is pressed
                answer = raw_input("\nDo you really want to exit ([y]/n)? ")
                if answer not in ["y", ""]:
                    continue
                self.stop()
                return
            except KeyboardInterrupt:  # occures when Ctrl+C is pressed
                self.stop()
                return

            item, args = self._get_item_by_line(line_input)
            if item is not None:
                f = item.get_function()
                if f is not None:
                    f(item, args, line_input)
                else:
                    Colorful.out.bold_red("command cannot be invoked")
            else:
                Colorful.out.bold_red("command cannot be found")

    def stop(self):
        # trigger 'on.before_stop' event
        self.obs.trigger("before_stop", self)
        if self._history_file is not None:
            readline.write_history_file(self._history_file)
        self._started = False
        # trigger 'on.after_stop' event
        self.obs.trigger("after_stop", self)

    def _complete(self, line, state):
        if state == 0:
            self._matches = []
            original_line = readline.get_line_buffer()

            for i in self._items:
                for r in i.complete(original_line):
                    self._matches.append(r)
        m = self._matches[state]
        if isinstance(m, CliItem):
            return m.get_completion_name()
        return m

    def _get_item_by_line(self, line):
        for i in self._items:
            item, args = i.get_item_by_line(line)
            if item is not None and item.is_enabled():
                return item, args
        return None, line

    def set_prompt(self, prompt):
        self._prompt = prompt

    def set_welcome_text(self, welcome_text):
        self._welcome_text = welcome_text

    def get_items(self):
        return self._items

    def register_item(self, item):
        self._items.append(item)

    def clear_items(self):
        for i in self._items:
            self._items.remove(i)

    def remove_item(self, name):
        for i in self._items:
            if i.get_name() == name:
                self._items.remove(i)

    def enable_items_by_category(self, category):
        for i in self._items:
            i.set_enabled(i.in_category(category))



# -*- coding: utf-8 -*-

import os
import re


class CliItem(object):
    def __init__(self, name, function=None, value=None, enabled=True, subitems=[], tab_delimiter=" ", categories=[]):
        self._name = name
        self._function = function
        self._value = value
        self._enabled = enabled
        self._subitems = subitems
        self._tab_delimiter = tab_delimiter
        self._categories = categories

    def get_name(self):
        return self._name

    def get_completion_name(self):
        return self._name + self._tab_delimiter

    def get_function(self):
        return self._function

    def get_value(self):
        return self._value

    def is_enabled(self):
        return self._enabled

    def set_enabled(self, enabled):
        self._enabled = enabled

    def in_category(self, category):
        return category in self._categories

    def get_item_by_line(self, line):
        if line.startswith(self.get_completion_name()):
            item, args = line.split(self._tab_delimiter, 1)
            if not args:
                return self, args
            else:
                for i in self._subitems:
                    subitem, subargs = i.get_item_by_line(args)
                    if subitem is not None:
                        if subitem.get_function() is not None:
                            return subitem, subargs
                        else:
                            return self, args
                return self, args
        elif self._name == line:
            return self, ""
        else:
            return None, line

    def complete(self, line):
        matches = []
        if self._enabled:
            if line.startswith(self.get_completion_name()):
                item, args = line.split(self._tab_delimiter, 1)
                for s in self._subitems:
                    for i in s.complete(args):
                        matches.append(i)
            elif self.get_completion_name().startswith(line):
                matches.append(self)
        return matches

    def register_subitem(self, subitem):
        self._subitems.append(subitem)

    def get_help(self):
        arguments = ""
        description = self._function.__doc__ or "No description available"
        if self._function and self._function.__doc__ is not None:
            help_str = self._function.__doc__
            arguments_match = re.search("(?:arguments|args):(.*)", help_str)
            description_match = re.search("(?:description|desc):(.*)", help_str)

            if arguments_match:
                arguments = arguments_match.groups()[0].strip()
            if description_match:
                description = description_match.groups()[0].strip()

        return {"arguments": arguments, "description": description}


class CliSysPathItem(CliItem):
    def __init__(self):
        CliItem.__init__(self, "path_item")

    def _listdir(self, root):
        matches = []
        for name in os.listdir(root):
            path = os.path.join(root, name)
            if os.path.isdir(path):
                name += os.sep
            matches.append(name)
        return matches

    def _complete_path(self, path):
        matches = []
        if path == "":
            return self._listdir(".")
        dirname, filename_part = os.path.split(path)
        tmp_dirname = dirname if dirname else "."
        matches = [p for p in self._listdir(tmp_dirname) if p.startswith(filename_part)]
        if len(matches) > 1:
            return matches
        if os.path.isdir(os.path.join(dirname, matches[0])):
            return matches
        return [matches[0] + self._tab_delimiter]

    def complete(self, line):
        return self._complete_path(line)



# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    name="clicore",
    description="cli core library for python",
    version="0.00.03",
    author="Timo Furrer",
    author_email="tuxtimo@gmail.com",
    url="https://github.com/timofurrer/cli",
    install_requires=["observable==0.00.03", "colorful==0.00.02"],
    packages=["clicore"]
)
